import cv2
import pytest
import Levenshtein
import numpy as np
from typing import List
from pathlib import Path
from fastanpr import FastANPR

fastanpr = FastANPR()


@pytest.mark.asyncio
async def test_image_3d():
    file = Path(__file__).parent / 'images/image001.jpg'
    expected_plates = ['BVH826']
    predicted_plates = (await fastanpr.run(image_path_to_ndarray(str(file))))[0]

    # same number of plates predicted
    assert len(predicted_plates) == len(expected_plates)

    # ocr box is inside detected box
    assert all(
        ocr_box_encapsulates_detected_box(poly_to_bbox(predicted_plate.rec_poly), predicted_plate.det_box)
        for predicted_plate in predicted_plates
    )

    # maximum Levenshtein distance must be 1
    assert all(
        min(Levenshtein.distance(predicted_plate.rec_text, expected_plates) for expected_plates in expected_plates) <= 1
        for predicted_plate in predicted_plates
    )


@pytest.mark.asyncio
@pytest.mark.parametrize(
    'file, expected_plates', [
        (Path(__file__).parent / 'images/image001.jpg', ['BVH826']),
        (Path(__file__).parent / 'images/image002.jpg', ['XJIIMES', 'FEIIDEK']),
        (Path(__file__).parent / 'images/image003.jpg', ['CLS7145', '319ZBA']),
        (Path(__file__).parent / 'images/image004.jpg', ['560ZPC']),
        (Path(__file__).parent / 'images/image005.jpg', ['ERS73W']),
        (Path(__file__).parent / 'images/image006.jpg', ['BPN325']),
    ]
)
async def test_image_4d(file: str, expected_plates: List[str]):
    predicted_plates = (await fastanpr.run([image_path_to_ndarray(str(file))]))[0]

    # same number of plates predicted
    assert len(predicted_plates) == len(expected_plates)

    # ocr box is inside detected box
    assert all(
        ocr_box_encapsulates_detected_box(poly_to_bbox(predicted_plate.rec_poly), predicted_plate.det_box)
        for predicted_plate in predicted_plates
    )

    # minimum Levenshtein distance must be 1
    assert all(
        min(Levenshtein.distance(predicted_plate.rec_text, expected_plates) for expected_plates in expected_plates) <= 1
        for predicted_plate in predicted_plates
    )


def ocr_box_encapsulates_detected_box(ocr_box: List[int], det_box: List[int]) -> bool:
    return (ocr_box[0] >= det_box[0] and ocr_box[1] >= det_box[1] and
            ocr_box[2] <= det_box[2] and ocr_box[3] <= det_box[3])


def poly_to_bbox(poly: List[List[int]]) -> List[int]:
    x_coordinates = [point[0] for point in poly]
    y_coordinates = [point[1] for point in poly]
    x_min = min(x_coordinates)
    y_min = min(y_coordinates)
    x_max = max(x_coordinates)
    y_max = max(y_coordinates)
    return [x_min, y_min, x_max, y_max]


def image_path_to_ndarray(file: str) -> np.ndarray:
    return cv2.cvtColor(cv2.imread(file), cv2.COLOR_BGR2RGB)



from typing import List
from pydantic import BaseModel


class NumberPlate(BaseModel):
    det_box: List[int]
    det_conf: float
    rec_poly: List[List[int]] = None
    rec_text: str = None
    rec_conf: float = None

    class Config:
        frozen = True



import os
import sys
import subprocess
from setuptools import setup, find_packages

# Getting build version
try:
    VERSION = (
        subprocess.run(["git", "describe", "--tags"], stdout=subprocess.PIPE)
        .stdout.decode("utf-8")
        .strip()
        .lstrip("v.")
    )

    if "-" in VERSION:
        v, i, s = VERSION.split("-")
        VERSION = v + "+" + i + ".git." + s

    assert "-" not in VERSION
    assert "." in VERSION

    assert os.path.isfile("fastanpr/version.py")
    with open("fastanpr/version.py", "w", encoding="utf-8") as fh:
        fh.write(f"__version__ = \"{VERSION}\"\n")

except Exception:
    with open("fastanpr/version.py", "r", encoding="utf-8") as fh:
        VERSION = fh.read().strip().lstrip("__version__ = \"").rstrip("\"")

# Set Python version requirements based on the operating system
if sys.platform.startswith('win'):
    PYTHON_REQUIRES = '>=3.8, <3.11'
else:
    PYTHON_REQUIRES = '>=3.6'

# Getting long description from README
with open("README.md", "r") as fh:
    LONG_DESCRIPTION = fh.read()

setup(
    name="fastanpr",
    version=VERSION,
    author="arvindrajan92 (Arvind Rajan)",
    author_email="<arvindrajan92@gmail.com>",
    description='A fast automatic number-plate recognition (ANPR) library',
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    package_data={'': ['*.pt'], 'fastanpr': ['*.pt']},
    include_package_data=True,
    install_requires=['ultralytics>=8.1.34', 'paddlepaddle>=2.6.1', 'paddleocr==2.7.2', 'pydantic>=2.6.4'],
    python_requires=PYTHON_REQUIRES,
    extras_require={
        'dev': ['pytest', 'pytest-asyncio', 'twine', 'python-Levenshtein', 'setuptools', 'wheel', 'twine', 'flake8']
    },
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Operating System :: OS Independent",
    ],
    url='https://github.com/arvindrajan92/fastanpr',
    keywords=[
        'python', 'anpr', 'fast', 'licence plate', 'number plate', 'detection', 'recognition', 'YOLOv8', 'paddleocr',
        'paddlepaddle'
    ]
)
